var moment = require("moment");
var RodcoModels = require("../");
var errors = require("throw.js");

function Model(){};
Model.table_name = "contact"

Model.create = function(contact){
  contact.updates = {list: []}
  contact.updated_by = RodcoModels.user.id;
  contact.created_by = RodcoModels.user.id;
  contact.namespace_id = RodcoModels.user.namespace_id;
  return RodcoModels.knex.table(Model.table_name)
  .insert(contact)
  .returning("*")
  .then(function(results){
    return results[0];
  })
}

Model.updateSecurity = function(contact){
  var contactToUpdate = { ACL: contact.ACL, profile: contact.profile, id: contact.id }
  return Model.update( contactToUpdate );
}

Model.update = function(deltaContact){
  console.log(deltaContact)
  delete deltaContact.namespace_id;
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, id: deltaContact.id, active: true})
  .then( function(results){
    if(results.length == 0 ) throw new errors.NotFound("Contact does not exists or is not active. Code 1")
    var contact = results[0];
    deltaContact.updates = contact.updates;
    if(!deltaContact.updates) deltaContact.updates = {list: []}
    var keys = Object.keys(deltaContact);
    keys.forEach( function(key){
      if( deltaContact[key] != contact[key] ) contact.updates.list.push({ user: RodcoModels.user.name, key: key, old: ""+contact[key], current: ""+deltaContact[key] })
    })
    keys = Object.keys(contact.ids||{});
    keys.forEach(function(key){
      if( !deltaContact.ids[key] && contact.ids[key] ) deltaContact.ids[key] = contact.ids[key];
    })
    deltaContact.updates = contact.updates;
    deltaContact.updated_at = moment();
    deltaContact.updated_by = RodcoModels.user.id;
    return RodcoModels.knex.table(Model.table_name)
    .update(deltaContact)
    .returning("*")
    .where({id: deltaContact.id, namespace_id: RodcoModels.user.namespace_id})
  })
  .then( function(res){
    return res[0];
  })
}



Model.delete = function(id){
  var updateValues = {
    updated_at: moment(),
    updated_by: RodcoModels.user.id,
    active: false
  }
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, id: id, active: true})
  .then( function(results){
    if(results.length == 0 ) throw new errors.NotFound("Contact does not exists or is not active.")
    updateValues.updates = results[0].updates;
    updateValues.updates.list.push({user: RodcoModels.user.name, key: "active", old: "true", current: "false"})
    return RodcoModels.knex.table(Model.table_name)
    .update(updateValues)
    .where({id: id, namespace_id: RodcoModels.user.namespace_id})
  })
  .then( function(){
    return {};
  })
}

Model.all = function(){
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, active: true })
}

Model.allByIds = function(ids){
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, active: true })
  .whereIn("id",ids)
}

Model.one = function(id){
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({id: id })
  .then(function(results){
    if(!results || results.length == 0) throw new errors.NotFound();
    return results[0];
  })
}

Model.getSecurityProfile = function(id){
  return RodcoModels.knex.table(Model.table_name)
  .select(["id","ACL","profile"])
  .where({id: id })
  .then(function(results){
    if(!results || results.length == 0) throw new errors.NotFound();
    return results[0];
  })
}


Model.getByPhone = function(phone, throw404){
  return RodcoModels.knex.table("contact")
  .select("*")
  .where({"mobile_phone": phone, namespace_id: RodcoModels.user.namespace_id,})
  .then(function(results){
    if(!results || results.length == 0 && throw404) throw new errors.NotFound();
    else if(!results || results.length == 0) return null;
    return results[0];
  })
}

Model.getByEmail = function(email, throw404){
  return RodcoModels.knex.table("contact")
  .select("*")
  .where({"email": email, namespace_id: RodcoModels.user.namespace_id})
  .then(function(results){
    if(!results || results.length == 0 && throw404) throw new errors.NotFound();
    if(!results || results.length == 0) return null;
    return results[0];
  })
}

Model.getRecentOfCreator = function(){
  return RodcoModels.knex.table("contact")
  .select("*")
  .where({"created_by": RodcoModels.user.id, namespace_id: RodcoModels.user.namespace_id})
  .orderBy("created_at","desc")
  .then(function(results){
    return results;
  })
}

module.exports= Model;
