var moment = require("moment");
var RodcoModels = require("../../");
var errors = require("throw.js");

function Model(){};
Model.table_name = "catalog_product"

Model.create = function(product){
  product.updates = {list: []};
  if(!product.photos) product.photos = {list: []};
  if(!product.attributos) product.attributos = {list: []};
  if(!product.bullets) product.bullets = {list: []};

  product.updated_by = RodcoModels.user.id;
  product.namespace_id = RodcoModels.user.namespace_id;
  return RodcoModels.knex.table(Model.table_name)
  .insert(product)
  .returning("*")
  .then(function(results){
    return results[0];
  })
}

Model.update = function(deltaContact){
  console.log(deltaContact)
  delete deltaContact.namespace_id;
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, id: deltaContact.id, active: true})
  .then( function(results){
    if(results.length == 0 ) throw new errors.NotFound("Catalog Product does not exists or is not active. Code 1")
    var product = results[0];
    deltaContact.updates = product.updates;
    if(!deltaContact.updates) deltaContact.updates = {list: []}
    var keys = Object.keys(deltaContact);
    keys.forEach( function(key){
      if( deltaContact[key] != product[key] ) product.updates.list.push({ user: RodcoModels.user.name, key: key, old: ""+product[key], current: ""+deltaContact[key] })
    })
    keys = Object.keys(product.ids||{});
    keys.forEach(function(key){
      if( !deltaContact.ids[key] && product.ids[key] ) deltaContact.ids[key] = product.ids[key];
    })
    deltaContact.updates = product.updates;
    deltaContact.updated_at = moment();
    deltaContact.updated_by = RodcoModels.user.id;
    return RodcoModels.knex.table(Model.table_name)
    .update(deltaContact)
    .returning("*")
    .where({id: deltaContact.id, namespace_id: RodcoModels.user.namespace_id})
  })
  .then( function(res){
    return res[0];
  })
}


Model.delete = function(id){
  var updateValues = {
    updated_at: moment(),
    updated_by: RodcoModels.user.id,
    active: false
  }
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, id: id})
  .then( function(results){
    if(results.length == 0 ) throw new errors.NotFound("Product does not exists or is not active.")
    updateValues.updates = results[0].updates;
    updateValues.updates.list.push({user: RodcoModels.user.name, key: "active", old: "true", current: "false"})
    return RodcoModels.knex.table(Model.table_name)
    .update(updateValues)
    .where({id: id, namespace_id: RodcoModels.user.namespace_id})
  })
  .then( function(){
    return {};
  })
}

Model.all = function(){
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, active: true })
}

Model.one = function(id){
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({id: id })
  .then(function(results){
    if(!results || results.length == 0) throw new errors.NotFound();
    return results[0];
  })
}


module.exports= Model;
