var moment = require("moment");
var RodcoModels = require("../../");
var errors = require("throw.js");

function Model(){};
Model.table_name = "project_buyer";

Model.create = function(model){
  model.namespace_id = RodcoModels.user.namespace_id;

  return RodcoModels.knex.table(Model.table_name)
  .select("id")
  .where({namespace_id: RodcoModels.user.namespace_id, project_opportunity_id: model.project_opportunity_id, contact_id: model.contact_id})
  .then(function(results){
    if( results.length > 0 ) return errors.Conflict("Contact is already asigned to this project");
    return RodcoModels.knex.table(Model.table_name)
    .insert(model)
    .returning("*")
  })
  .then(function(results){
    return results[0];
  })
}

Model.createContactAndBuyer = function(model){
  model.namespace_id = RodcoModels.user.namespace_id;
  var project_opportunity_id = model.project_opportunity_id;
  var contact;
  delete model.project_opportunity_id;
  RodcoModels.contact.create(model)
  .then(function(contactResult){
    contact = contactResult;
    return RodcoModels.knex.table(Model.table_name)
    .insert({contact_id: contactResult.id, project_opportunity_id: project_opportunity_id})
    .returning("*")
  })
  .then(function(results){
    return {contact: contactResult, buyer: results[0]}
  })
}


Model.update = function(delta){
  delete delta.namespace_id;
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, id: delta.id})
  .then( function(results){
    if(results.length == 0 ) throw new errors.NotFound("Opportunity does not exists")
    return RodcoModels.knex.table(Model.table_name)
    .update(delta)
    .returning("*")
    .where({id: delta.id, namespace_id: RodcoModels.user.namespace_id})
  })
  .then( function(res){
    return res[0];
  })
}


Model.delete = function(id){
  return RodcoModels.knex.table(Model.table_name)
  .delete("*")
  .where({namespace_id: RodcoModels.user.namespace_id, id: id})
  .then( function(){
    return {};
  })
}

Model.one = function(id, throwErrorOn404){
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id,id: id})
  .then(function(results){
    if(!results || results.length == 0 && throwErrorOn404) throw new errors.NotFound();
    else if(!results || results.length == 0 && !throwErrorOn404) return null;
    return results[0];
  })
}

Model.all = function(id){
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id})
}

Model.byOpportunity = function(project_opportunity_id){
  var buyers = [];
  var buyerMap = {};
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, project_opportunity_id: project_opportunity_id})
  .then( function(buyers){
    var contactIds = buyers.map(function(buyer){
      buyerMap[buyer.contact_id] = buyer;
      return buyer.contact_id;
    })
    return RodcoModels.contact.allByIds(contactIds)
  })
  .then( function(contacts){
    return contacts.map( function(contact){
      buyerMap[contact.id].contact = contact;
      return buyerMap
    })
  })
}




module.exports= Model;


