var moment = require("moment");
var RodcoModels = require("../../");
var errors = require("throw.js");

function Model(){};
Model.table_name = "project_opportunity";

Model.create = function(model){
  model.namespace_id = RodcoModels.user.namespace_id;
  model.updated_by = RodcoModels.user.id;
  return RodcoModels.knex.table(Model.table_name)
  .insert(model)
  .returning("*")
  .then(function(results){
    return results[0];
  })
}

Model.touchUpdated_at = function(model){
  return RodcoModels.knex.table(Model.table_name)
  .update({ updated_at: moment() })
  .returning("*")
  .where({ id: model.id, namespace_id: RodcoModels.user.namespace_id })
}

Model.update = function(delta ){
  delete delta.namespace_id;
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id, id: delta.id, active: true})
  .then( function(results){
    if(results.length == 0 ) throw new errors.NotFound("Opportunity does not exists or is not active. Code 1")
    var oppy = results[0];
    delta.updates = oppy.updates;
    if(!delta.updates) delta.updates = {list: []}
    var keys = Object.keys(delta);
    keys.forEach( function(key){
      if( delta[key] != oppy[key] ) delta.updates.list.push({ user: RodcoModels.user.name, key: key, old: ""+oppy[key], current: ""+delta[key] })
    })
    keys = Object.keys(oppy.ids||{});
    keys.forEach(function(key){
      if( !delta.ids[key] && oppy.ids[key] ) delta.ids[key] = oppy.ids[key];
    })
    delta.updated_at = moment();
    delta.updated_by = RodcoModels.user.id;
    return RodcoModels.knex.table(Model.table_name)
    .update(delta)
    .returning("*")
    .where({id: delta.id, namespace_id: RodcoModels.user.namespace_id})
  })
  .then( function(res){
    return res[0];
  })
}

Model.updateRemindInDays = function(models){
  return RodcoModels.knex.batchUpdate(Model.table_name,models,[remind_in_days]);
}

Model.delete = function(id){
  return RodcoModels.knex.table(Model.table_name)
  .delete("*")
  .where({namespace_id: RodcoModels.user.namespace_id, id: id})
  .then( function(){
    return {};
  })
}

Model.one = function(id, throwErrorOn404){
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id,id: id})
  .then(function(results){
    if(!results || results.length == 0 && throwErrorOn404) throw new errors.NotFound();
    else if(!results || results.length == 0 && !throwErrorOn404) return null;
    return results[0];
  })
}

Model.all = function(id){
  return RodcoModels.knex.table(Model.table_name)
  .select("*")
  .where({namespace_id: RodcoModels.user.namespace_id})
}





module.exports= Model;


