//During the test the env variable is set to test
var Helper = require("./helper");

//Require the dev-dependencies
var chai = require('chai');
var BbPromise = require("bluebird");
var should = chai.should();
var knexfile = require("../knexfile")
var RodcoModels = require("../");
var Model;
var contact;

//Our parent block
describe('Contact', () => {

  before(function (done) {
    Helper.setupTest(false,knexfile)
    .then(function(){
      Model  = RodcoModels.contact;
      done();
    })
  });

  after(function (done) {
    Helper.clear()
        .then(function(){
          done();
        })
  });

    it('it should create a contact', function(done){

      Model.create({first_name: "test",last_name:"test","email":"thistest@test.com","mobile_phone":"123"})
      .then( function(contactResult){
        contact = contactResult;
        contact.id.should.not.equal(null);
        done();
      })
    });

    it('it should get one contact', function(done){
      Model.one(contact.id)
      .then( function(contact){
        contact.id.should.not.equal(null);
        done();
      })
    });

    it('it should get one contact by  email', function(done){
      Model.getByEmail("thistest@test.com")
      .then( function(contact){
        contact.id.should.not.equal(null);
        done();
      })
    });

    it('it should throw error by email', function(done){
      Model.getByEmail("error@test.com",true)
      .catch(function(){
        done();
      })
    });

    it('it should get one contact by phone', function(done){
      Model.getByPhone("123")
      .then( function(contact){
        contact.id.should.not.equal(null);
        done();
      })
    });

    it('it should throw error by phone', function(done){
      Model.getByPhone("121212",true)
      .catch( function(err){
        done();
      })
    });

    it('it should update security of one contact', function(done){
      contact.profile = "ABC";
      contact.ACL = {doSomething:true};
      Model.updateSecurity(contact)
      .then( function(contact){
        contact.profile.should.equal("ABC");
        contact.ACL.doSomething.should.equal(true);
        done();
      })
    });

    it('it should delete a contacto', function(done){

      Model.delete(contact.id)
      .then( function(contacts){
        return Model.all();
      })
      .then(function(res){
        res.length.should.equal(0);
        done();
      })
    });

});
